import ctypes
import ujson

from edge import eventlog

_LIB_HASP_ADMIN_API = 'libsntl_adminapi_linux_x86_64.so'


def get_hasp_license_manager_info():

    query_text = ''.join(
        ['<admin>',
         '<license_manager format="json">',
         '<element name="*"/>',
         '</license_manager>',
         '</admin>'])

    raw_hasp_key_jnode_list = _get_admin_api_reply(query_text)
    hasp_license_manager_info = ujson.loads(raw_hasp_key_jnode_list)[:-1]  # strip summary
    return hasp_license_manager_info


def _get_admin_api_reply(query_text, vendor_id=None):
    """
    :type query_text: str
    :type vendor_id: str | None
    :rtype: str
    """
    _HOST = 'localhost'

    admin_api = ctypes.CDLL(_LIB_HASP_ADMIN_API)

    session_scope = ''.join(['<haspscope>',
                             '<host>%s</host>' % _HOST,
                             '</haspscope>'])

    query_scope = ''
    if vendor_id:
        query_scope = query_scope.join(['<haspscope>',
                                        '<vendor><id>%s</id></vendor>' % vendor_id,
                                        '</haspscope>'])

    session_context = ctypes.POINTER(ctypes.c_void_p)()
    rc = admin_api.sntl_admin_context_new_scope(ctypes.byref(session_context), session_scope)
    if rc:
        eventlog.log_d("fail: sntl_admin_context_new_scope (%s)" % rc)
        raise RuntimeError(rc)

    try:
        reply_buffer = ctypes.POINTER(ctypes.c_char_p)()
        rc = admin_api.sntl_admin_get(session_context, query_scope, query_text, ctypes.byref(reply_buffer))
        if rc:
            eventlog.log_d("fail: sntl_admin_get (%s)" % rc)
            raise RuntimeError(rc)

        try:
            raw_reply_text = ctypes.cast(reply_buffer, ctypes.c_char_p).value
        finally:
            admin_api.sntl_admin_free(reply_buffer)

    finally:
        admin_api.sntl_admin_context_delete(session_context)

    return _strip_reply_comments(raw_reply_text)


def _strip_reply_comments(raw_reply_text):
    """
    Strip comments in JSON-string and make string a JSON-array
    :type raw_reply_text: str
    :rtype: str
    """
    begin_comment = 0
    end_comment = 0

    while begin_comment != -1 and end_comment != -1:
        begin_comment = raw_reply_text.find('/*')
        end_comment = raw_reply_text.find('*/')
        if begin_comment == -1 and end_comment == -1:
            continue
        s = raw_reply_text[:begin_comment] + '' + raw_reply_text[end_comment + 2:]
        raw_reply_text = s

    return '[' + raw_reply_text + ']'
